// CForth LowPower project ***************************************************
//
// Author and (c)2020-24 Wolfgang Schemmert. The copyright claim is restricted
// to this file and the depending files "main.h","cforthLP.c","cforthLP.h",
// and "globaldefines.h"
// Other depending material is published, copyrighted and licensed
// by STMicroelectronics and Segger GmbH
//
// Contact: <www.midi-and-more.de>
// A detailled hardware construction and operation manual
// is provided at this website
//
// ****************************************************************************
//      This program is free software: you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation, either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//      Licence: GPL-3
//
// ****************************************************************************

#include "stm32f0xx.h"
#include "globaldefines.h"
#include "main.h"

int main(void)    // ********************************************************
// **************************************************************************
{

START:

    RCC->AHBENR |= RCC_AHBENR_GPIOAEN;  // Enable Port A clock
    RCC->AHBENR |= RCC_AHBENR_GPIOBEN;  // Enable Port B clock

//	initGPIO();
    GPIOA->MODER = 0x00000000;      //primarily all input
    GPIOB->MODER = 0x00000000;      //primarily all input
    GPIOA->PUPDR = 0xAAAAAAAA;      //primarily all inputs pulldown 
    GPIOB->PUPDR = 0xAAAAAAAA;      //primarily all inputs pulldown

#ifdef LQFP32
//pin config repeated specifically:
  #ifdef LED_PB0
 	GPIOB->PUPDR &= ~(GPIO_PUPDR_PUPDR0);	//pull-down off, wasts current
    GPIOB->MODER &= ~(GPIO_MODER_MODER0);
    GPIOB->MODER |= GPIO_MODER_MODER0_0 ;	//set PortB0 (LED) as output
    GPIOB->BSRR =  GPIO_BSRR_BS_0;			//PB0 (LED) OFF
  #endif
  #ifdef MODUL_24
	GPIOA->PUPDR &= ~(GPIO_PUPDR_PUPDR11);	//pull-down off, wasts current
    GPIOA->MODER &= ~(GPIO_MODER_MODER11);
    GPIOA->MODER |= GPIO_MODER_MODER11_0 ;	//set PortA11 (LED) as output
    GPIOA->BSRR =  GPIO_BSRR_BS_11;			//Pa11 (LED) OFF
  #endif
  #ifdef LED_PB8
 	GPIOB->PUPDR &= ~(GPIO_PUPDR_PUPDR8);	//pull-down off, wasts current
    GPIOB->MODER &= ~(GPIO_MODER_MODER8);
    GPIOB->MODER |= GPIO_MODER_MODER8_0 ;	//set PortB8 (LED) as output
    GPIOB->BSRR =  GPIO_BSRR_BS_8;			//Pa8 (LED) OFF
  #endif
#else
//pin config repeated specifically:
  #ifdef MODUL_R
	GPIOB->PUPDR &= ~(GPIO_PUPDR_PUPDR1);	//pull-down off, wasts current
    GPIOB->MODER &= ~(GPIO_MODER_MODER1);
    GPIOB->MODER |= GPIO_MODER_MODER1_0 ;	//set PortB1 (LED) as output
    GPIOB->BSRR =  GPIO_BSRR_BS_1;			//PB1 (LED) OFF
  #else	//LEDPB8
	GPIOB->PUPDR &= ~(GPIO_PUPDR_PUPDR8);	//pull-down off, wasts current
    GPIOB->MODER &= ~(GPIO_MODER_MODER8);
    GPIOB->MODER |= GPIO_MODER_MODER8_0 ;	//set PortB8 (LED) as output
    GPIOB->BSRR =  GPIO_BSRR_BS_8;			//13March21:PB8 (LED) OFF
  #endif
#endif

#ifdef CLOCK_3M68
	gfSpeed = 4;	//default = 921kHz
#endif
#ifdef CLOCK_14M
	gfSpeed = 16;	//default = 921kHz
#endif
#ifdef CLOCK_48M
	gfSpeed = 1;	//default = 48MHz
#endif

	SetSysClock();


    gLEDCOUNT = 0;
    gDELAYCOUNT = 0;
    gTIMEOUT = 0;
    gBaud = DEFAULT_BAUD;
    gBaudSet = 0;
	gfNumBase = DEFAULT_NUMBASE;
	gfTempNumBase = DEFAULT_NUMBASE;
	gLedBusy = 3;
	gfRandom = 314159265;		//seed for xorshift32 algorithm

    SysTick_Config(SystemCoreClock / (1000 * gfSpeed));	
								//1 ms for any SystemCoreClock
								//SystemCoreClock is derived from SYSCLOCK=HSE

    msDelay(50);

//init I/O state  *************************
    op_ABORT();					//reset default Forth system data
    popFlashPage25();			//loads only gBaud, gfNumBase and gProject

    msDelay(2);
 //init communication  ********************
    initUsart1();

    msDelay(2);
    sendTB(CRR);
    sendString(gTypeStr);
    sendTB(HARDWARE_VERSION + 0x30);
    sendTB('.');
    sendNS32(REVNUM);
    sendString(gCopyStr); 

    popFlashProject();				//if (gProject==0) start empty project
									//if virgin or gProject==0 emptyForth();

    while(1) {
		quit();
		if (query() == 0) break;
		if (gfTibMax == 0) continue;
		eval();
    }
    msDelay(100);
    goto START;					//force cold reset 
}

// ##########################################################################

// Misc Low Level Routines
//************************

#ifdef PLLCLOCK
void SetSysClock(void)  // ******************************************
// *************************************************************************
{
__IO uint32_t StartUpCounter = 0, HSEStatus = 0;

//SYSCLK, HCLK, PCLK configuration ----------------------------------------
//Enable HSE 
  RCC->CR |= ((uint32_t)RCC_CR_HSEON);

//Wait till HSE is ready and if Time out is reached exit 
  do  {
    HSEStatus = RCC->CR & RCC_CR_HSERDY;
    StartUpCounter++;
  } while((HSEStatus == 0) && (StartUpCounter != HSE_STARTUP_TIMEOUT));

  if ((RCC->CR & RCC_CR_HSERDY) != RESET)  { //RESET=0 defined in stm32f0xx.h
//Enable Prefetch Buffer and set Flash Latency 
#ifdef CLOCK_48M
    FLASH->ACR = FLASH_ACR_PRFTBE | FLASH_ACR_LATENCY;
#else
	FLASH->ACR = FLASH_ACR_PRFTBE; // | FLASH_ACR_LATENCY;=zero waitstates
#endif
	RCC->CFGR &= ~(RCC_CFGR_HPRE);		//clear all HPRE options
	if (gfSpeed == 16)  RCC->CFGR |=  RCC_CFGR_HPRE_DIV16;
	else if (gfSpeed == 4)  RCC->CFGR |=  RCC_CFGR_HPRE_DIV4;
	else  RCC->CFGR |=  RCC_CFGR_HPRE_DIV1;

//PCLK = HCLK 
    RCC->CFGR |= (uint32_t)RCC_CFGR_PPRE_DIV1;

	RCC->CFGR &= (uint32_t)((uint32_t)~(RCC_CFGR_PLLSRC | 
									RCC_CFGR_PLLXTPRE | RCC_CFGR_PLLMULL));
#ifdef  CLOCK_48M
//PLL configuration = HSE with 8.0 MHz: HSE * 6 * 48MHz
    RCC->CFGR |= (uint32_t)(RCC_CFGR_PLLSRC_PREDIV1 | 
							RCC_CFGR_PLLXTPRE_PREDIV1 | RCC_CFGR_PLLMULL6);
#else
//PLL configuration = HSE with 3.68 MHz: HSE * 4 * 14.74MHz
    RCC->CFGR |= (uint32_t)(RCC_CFGR_PLLSRC_PREDIV1 | 
							RCC_CFGR_PLLXTPRE_PREDIV1 | RCC_CFGR_PLLMULL4);
#endif
//Enable PLL 
    RCC->CR |= RCC_CR_PLLON;

//Wait till PLL is ready
    while((RCC->CR & RCC_CR_PLLRDY) == 0) ;

//Select PLL as system clock source
    RCC->CFGR &= (uint32_t)((uint32_t)~(RCC_CFGR_SW));
    RCC->CFGR |= (uint32_t)RCC_CFGR_SW_PLL;

//Wait till PLL is used as system clock source
    while ((RCC->CFGR & (uint32_t)RCC_CFGR_SWS) != (uint32_t)RCC_CFGR_SWS_PLL) ;

	RCC->CFGR3 = RCC_CFGR3_USART1SW_0;	//SYSCLK selected for USART1
  }
}
// ************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE* //

#else

void SetSysClock(void)  // *************************************************
//modified by W.S. 27 Oct 2020
// *************************************************************************
{
__IO uint32_t StartUpCounter = 0, HSEStatus = 0;

//SYSCLK, HCLK, PCLK configuration ----------------------------------------
//reset PLLON and HSION bit
	RCC->CR &= ~(RCC_CR_PLLON | RCC_CR_HSION);
//Enable HSE
	RCC->CR |= ((uint32_t)RCC_CR_HSEON);

//Wait until HSE is ready and if Time out is reached exit
	do  {
		HSEStatus = RCC->CR & RCC_CR_HSERDY;
		StartUpCounter++;
	} while((HSEStatus == 0) && (StartUpCounter != HSE_STARTUP_TIMEOUT));

	if ((RCC->CR & RCC_CR_HSERDY) != RESET)	//then HSE is working, 
											//RESET=0 defined in stm32f0xx.h
	{
//Enable Prefetch Buffer //and set Flash Latency
		FLASH->ACR = FLASH_ACR_PRFTBE; // | FLASH_ACR_LATENCY;=zero waitstates

		RCC->CFGR &= ~(RCC_CFGR_HPRE);		//clear all HPRE options
		if (gfSpeed == 16)  RCC->CFGR |= RCC_CFGR_HPRE_DIV16;
		else if (gfSpeed == 4)  RCC->CFGR |= RCC_CFGR_HPRE_DIV4;
		else if (gfSpeed == 2)  RCC->CFGR |= RCC_CFGR_HPRE_DIV2;
		else  RCC->CFGR |= RCC_CFGR_HPRE_DIV1;

//PCLK = HCLK 
		RCC->CFGR |= (uint32_t)RCC_CFGR_PPRE_DIV1;

		RCC->CR &= ~RCC_CR_PLLON;		//DISable the main PLL explicitly

//system clock configuration = HSE with	 1.8432MHz or 3.6864MHz  or 14.7456MHz
		RCC->CFGR &= ~(RCC_CFGR_PLLSRC | RCC_CFGR_PLLXTPRE | RCC_CFGR_PLLMUL);
		RCC->CFGR3 = RCC_CFGR3_USART1SW_0;	//SYSCLK selected for USART1
//Select HSE as system clock source
		RCC->CFGR &= ~RCC_CFGR_SW;
		RCC->CFGR |= RCC_CFGR_SW_HSE;
// Wait until HSE is used as system clock source
		while ((RCC->CFGR & RCC_CFGR_SWS) != RCC_CFGR_SWS_HSE) ;
	}
}
// ************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE* //
#endif  //PLLCLOCK or not

void msDelay(uint32_t count)   // *******************************************
//blocking delay by 'count' milliseconds
// **************************************************************************
{
    gDELAYCOUNT = count;
    while(gDELAYCOUNT)  servSystem();		//unconditionally in this case
}


void wait5us(void)   // *****************************************************
// blocking delay ca 5us
// **************************************************************************
{
uint32_t countdown;
#ifdef CLOCK_48M
	countdown = 51;
#else
	countdown = 2;
#endif
    while (countdown)   {
        countdown-- ;
    }
}


void servSystem(void)   // ***************************************************
//not very time critical but permanently repeated functions for system control
//exclusively called in pullTermRxBuf
//this way it is regularily served when idle waiting for external input
//but watchdog resets when it hangs somewhere else
//slowed down by SysTick to 50 times/second
// ***************************************************************************
{
	if (gSYSCOUNT)  return;		//countdown in SysTick_Handler()

//else if if (gSYSCOUNT==0 )50 Hz service interval:  *************************
    gSYSCOUNT = 50;         //restart 20Hz service interval

//LED control
	if (gLedBusy > 1)  {
		if (gLEDCOUNT) gLEDCOUNT--;
#ifdef LQFP32
  #ifdef LED_PB0
		if  (gLEDCOUNT==0) 	GPIOB->BSRR = GPIO_BSRR_BS_0;
											// Set bit PB0 (LED) ON again
  #endif
  #ifdef MODUL_24
		if  (gLEDCOUNT==0) 	GPIOA->BSRR = GPIO_BSRR_BS_11;	//LED ON again
  #endif
  #ifdef LED_PB8
		if  (gLEDCOUNT==0) 	GPIOB->BSRR = GPIO_BSRR_BS_8;	//LED ON again
  #endif
#else
  #ifdef MODUL_R
		if  (gLEDCOUNT==0) 	GPIOB->BSRR = GPIO_BSRR_BS_1;
											// Set bit PB1 (LED) ON again
  #else  //LED_PB8
		if  (gLEDCOUNT==0) 	GPIOB->BSRR = GPIO_BSRR_BS_8;
											// Set bit PB8 (LED) ON again
  #endif
#endif
	}
//as precaution repeated here:
#ifdef LQFP32
  #ifdef LED_PB0
    else if (gLedBusy == 0)  GPIOB->BSRR = GPIO_BSRR_BR_0;//LED always OFF
    else  GPIOB->BSRR = GPIO_BSRR_BS_0;		//gLedBusy=1: LED now ON
  #endif
  #ifdef MODUL_24
    else if (gLedBusy == 0)  GPIOA->BSRR = GPIO_BSRR_BR_11;//LED always OFF
    else  GPIOA->BSRR = GPIO_BSRR_BS_11;		//gLedBusy=1: LED now ON
  #endif
  #ifdef LED_PB8
    else if (gLedBusy == 0)  GPIOB->BSRR = GPIO_BSRR_BR_8;//LED always OFF
    else  GPIOB->BSRR = GPIO_BSRR_BS_8;		//gLedBusy=1: LED now ON
  #endif
#else
  #ifdef MODUL_R
    else if (gLedBusy == 0)  GPIOB->BSRR = GPIO_BSRR_BR_1;//LED always OFF
    else  GPIOB->BSRR = GPIO_BSRR_BS_1;		//gLedBusy=1: LED now ON
  #else  //LED_PB8
    else if (gLedBusy == 0)  GPIOB->BSRR = GPIO_BSRR_BR_8;//LED always OFF
    else  GPIOB->BSRR = GPIO_BSRR_BS_8;		//gLedBusy=1: LED now ON
  #endif
#endif
}

void popFlashPage25(void)   // **********************************************
//read project independent system parameters
// **************************************************************************
{
uint32_t * flashPtr;
uint32_t test;

	emptyForth();			//primarily init defaults - implies gProject = 0;

	flashPtr = (uint32_t *)(PAGE25);
	test = ~(*flashPtr);//virgin test can't be=0 if project was saved in Flash
	flashPtr++;		

	if (test == 0)  return;	//defaults already init

	else  {			//Flash is not virgin, Project seems to be stored
										//programmed first=read lower16bit
		gBaud = test & 0x000000FF;		//baud rate is project independent
		if (gBaud == 0) gBaud = DEFAULT_BAUD;
		gfNumBase =  (test >> 16) & 0x000000FF;
		if (gfNumBase == 0) gfNumBase = DEFAULT_NUMBASE;
		gfTempNumBase = gfNumBase;
	}
	test = ~(*flashPtr);
										//programmed first=read lower16bit
	gProject = test & 0x000000FF;		//if==0, "empty project" will be used
	gLedBusy =  (test >> 16) & 0x000000FF;
}


void pushFlashPage25(void)   //**********************************************
//save project independent system parameters permanently
// **************************************************************************
{
uint16_t *flashPtr;

//unlock flash
#define KEY1    0x45670123				//mandatory unlock key defined by STM
#define KEY2    0xCDEF89AB
    while ((FLASH->SR & FLASH_SR_BSY) !=0);//wait until no operation going on
    if ((FLASH->CR & FLASH_CR_LOCK) != 0) {
            FLASH->KEYR = KEY1;
            FLASH->KEYR = KEY2;
    }
//erase Page25
    FLASH->CR |= FLASH_CR_PER;
    FLASH->AR = PAGE25;
    FLASH->CR |= FLASH_CR_STRT;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) !=0)  FLASH->SR |= FLASH_SR_EOP;
    FLASH->CR &= ~FLASH_CR_PER;

    flashPtr = (uint16_t *)PAGE25;
	FLASH->CR |= FLASH_CR_PG;

    //4 half-words for system and peripheral parameters
	//flash virgin test, too
    *flashPtr = (uint16_t)(~(gBaud) & 0x0000FFFF);  
    flashPtr++;
										//program first=read lower16bit
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;

   *flashPtr = (uint16_t)(~(gfNumBase) & 0x0000FFFF);
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;

	*flashPtr = (uint16_t)(~(gProject) & 0x0000FFFF); 
    flashPtr++;
										//program first=read lower16bit
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;

   *flashPtr = (uint16_t)(~(gLedBusy) & 0x0000FFFF);
    flashPtr++;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
//terminate
    FLASH->CR &= ~FLASH_CR_PG;
//lock flash
    FLASH->CR |= FLASH_CR_LOCK;
    msDelay(10);
}


void popFlashProject(void)   //**********************************************
// **************************************************************************
{
uint32_t k, pwm;
uint32_t  *flashPtr;
uint32_t *sramPtr;

//emptyForth() already updated in popFlashPage23() (default system parameters)

	if (gProject)  {
		if (gProject == 1) flashPtr = (uint32_t *)PAGE26;
		else flashPtr = (uint32_t *)PAGE29;
//4 words=16bytes for system and peripherial parameters
		k = ~(*flashPtr);			//cannot be==0 if project saved in Flash
		flashPtr++;
		pwm =  ~(*flashPtr);		//cannot be==0 if project saved in Flash
		flashPtr++;

		if ((k | pwm ) == 0) {					//project Flash is virgin
			emptyForth();						//implies gProject=0
		}
		else   {
			gLedBusy = (k & 0x0000FFFF) - 1;
			gfNumBase = (k >> 16) & 0x0000000FF;
			gfTempNumBase = gfNumBase;			//overrides setup from PAGE25
			gfSpeed = (pwm >> 16) & 0x000000FF;

#ifdef CLOCK_3M68
			if ((gfSpeed != 1)&&(gfSpeed != 4)) gfSpeed = 4;//default 921.6kHz
#endif
#ifdef CLOCK_14M
			if ((gfSpeed !=1)&&(gfSpeed !=16)) gfSpeed = 16;//default 921.6kHz
#endif
#ifdef CLOCK_48M
			if ((gfSpeed != 1) && (gfSpeed != 16)) gfSpeed = 1;//default 48MHz
#endif

			gfAuto = (pwm & 0x0000FFFF);

			k = ~(*flashPtr);
			flashPtr++;
			gfSpiMode = (k >> 8) & 0x00000003;
			gfSpiBits = k & 0x00000003F;
			gfSpiClock = (k >> 16) & 0x00000FFFF;

			k = ~(*flashPtr);
			flashPtr++;
			gfBaseFreq = (k >> 16) & 0x00000FFFF;

#ifdef CLOCK_3M68
			gfFreq = 4 * gfBaseFreq / gfSpeed;
#endif
#ifdef CLOCK_14M
			gfFreq = 16 * gfBaseFreq / gfSpeed;
#endif
#ifdef CLOCK_48M
			gfFreq = 16 * gfBaseFreq / gfSpeed;
#endif
			pwm = (k >> 8) & 0x000000FF;
			if (pwm >= 0x7F) pwm = 0xFFFFFFFF;	
										//manual cast 8 bit uint to 32 bit int
			gfPWM1 = (int32_t)pwm;
			if (gfFreq) {
				subFREQ();
				subPWM1();
#ifndef MOTOR
				pwm = k & 0x000000FF;
				if (pwm >= 0x7F) pwm = 0xFFFFFFFF;
				gfPWM2 = (int32_t)pwm;
				subPWM2();
#endif
			}

//loading is possible over page limits.
//gfUserHeader[] and gfTcode[] occupy a coherent data space in flash
			sramPtr = (uint32_t *) &(gfUserHeader[0]);
			for (k = 0;k < (MAXUSER * 4);k++)  {	
										//380 words=1520 bytes=ca.1.5pages
				*sramPtr = ~(*flashPtr);
				sramPtr++;
				flashPtr++;
			}
			sramPtr = (uint32_t *) &(gfTCode[0]);
			for (k = 0;k < 0x180;k++)  {	//1536(0x600) bytes = 1.5 pages
				*sramPtr = ~(*flashPtr);
				sramPtr++;
				flashPtr++;
			}
		}
	}
	
//from here if(gProject != 0) or not:
    wait5us();
    op_ABORT();					//reset project Forth system data
	if (gfAuto)  {
//emergency brake to avoid AUTO deadlock
		if ((GPIOA->IDR & 0x400) == 0) gfAuto = 0;	//if PA10 (RX input)is low
	}
	sendTB(CRR);
    op_QUEST();
    if (gfAuto) {
        gfParsed = gfAuto - 1;	//-1 to distinguish from empty AUTOEXE == 0
        interpretUserToken();
    }
}


void pushFlashProject(void)   //*********************************************
// **************************************************************************
{
uint32_t k;
uint16_t  *flashPtr;
uint32_t *sramPtr;

	if ((gProject != 1) && (gProject != 2)) return;	//precaution

//unlock flash
#define KEY1    0x45670123      //mandatory unlock key defined by STM
#define KEY2    0xCDEF89AB
    while ((FLASH->SR & FLASH_SR_BSY) !=0);
								//wait until no operation is going on
    if ((FLASH->CR & FLASH_CR_LOCK) != 0) {
            FLASH->KEYR = KEY1;
            FLASH->KEYR = KEY2;
    }

//erase Page26 (gProject==1) or Page29 (gProject==2)
    FLASH->CR |= FLASH_CR_PER;
    if (gProject == 1) FLASH->AR = PAGE26;
	else FLASH->AR = PAGE29;
    FLASH->CR |= FLASH_CR_STRT;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) !=0)  FLASH->SR |= FLASH_SR_EOP;
    FLASH->CR &= ~FLASH_CR_PER;

    FLASH->CR |= FLASH_CR_PG;
//write system parameters and gfUserHeader[] into page29
//programming Flash is possible only by 16bit half-words
//first programmed 16bit half-word will be read as lower 16bits
//                                         when data are read as 32bit words!!
//second programmed 16bit half-word will be read as higher 16bits
//                                         when data are read as 32bit words!!

	if (gProject == 1) flashPtr = (uint16_t *)PAGE26;
	else flashPtr = (uint16_t *)PAGE29;

    //4 words=16bytes for system and peripheral parameters
//    *flashPtr = (uint16_t)(~(2) & 0x0000FFFF);//read = lower 16bit
//    *flashPtr = 0;//read = lower 16bit
    *flashPtr = (uint16_t)(~(gLedBusy + 1) & 0x0000FFFF);//read = lower 16bit
							//cannot be zero, used for Flash virgin test
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
    flashPtr++;

	*flashPtr = (uint16_t)(~gfNumBase & 0x0000FFFF);//read = higher 16bit
    						//cannot be zero, used for Flash virgint test
	while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	flashPtr++;

    *flashPtr = (uint16_t)(~gfAuto & 0x0000FFFF);//read = lower 16bit
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
    flashPtr++;

   *flashPtr = (uint16_t)(~gfSpeed & 0x0000FFFF);//program last=read upper16bit
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
    flashPtr++;

	k =  ((gfSpiMode) << 8) | (gfSpiBits & 0xFF);
	*flashPtr = (uint16_t)(~k & 0x0000FFFF);  //program first=read lower16bit
	while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	flashPtr++;
	
	k =  gfSpiClock;
	*flashPtr = (uint16_t)(~k & 0x0000FFFF);  //program last=read upper16bit
	while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	flashPtr ++;	

#ifdef MOTOR
	k =  (((uint32_t)gfPWM1 & 0xFF)<< 8);
#else
	k =  ((((uint32_t)gfPWM1 & 0xFF)<< 8) | ((uint32_t)gfPWM2 & 0xFF));
#endif
	*flashPtr = (uint16_t)(~k & 0x0000FFFF);  //program first=read lower16bit
	while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	flashPtr++;

//bugfix 18Dec23 :gfBaseFreq instead of gfFreq
	*flashPtr = (uint16_t)(~gfBaseFreq&0x0000FFFF); //program last=read upper16bit
	while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
	if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
	flashPtr ++;	

//gfUserHeader[] and gfTcode[] occupy a coherent data space in flash
    sramPtr = (uint32_t *) &(gfUserHeader[0]);//here:252words=1008bytes+16=1024
    for (k = 0;k < 252;k++)  { //each entry 16bytes ->first 63 entries
        *flashPtr =(uint16_t)(~(*sramPtr) & 0x0000FFFF);
        while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
        if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
        flashPtr++;
        *flashPtr =(uint16_t)((~(*sramPtr) >> 16) & 0x0000FFFF);
        while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
        if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
        flashPtr++;
        sramPtr++;
    }
    FLASH->CR &= ~FLASH_CR_PG;

//erase Page27 (gProject==1) or Page30 (gProject==2)
    FLASH->CR |= FLASH_CR_PER;
	if (gProject == 1) FLASH->AR = PAGE27;
	else FLASH->AR = PAGE30;
    FLASH->CR |= FLASH_CR_STRT;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) !=0)  FLASH->SR |= FLASH_SR_EOP;
    FLASH->CR &= ~FLASH_CR_PER;

    FLASH->CR |= FLASH_CR_PG;
	if (gProject == 1) flashPtr = (uint16_t *)PAGE27;
	else flashPtr = (uint16_t *)PAGE30;
//write remaining part of gfUserHeader:MAXUSER=(95-63) *4=128 words total
    for (k = 0;k < 128;k++)  { //each entry 16bytes --> last 32entries
        *flashPtr =(uint16_t)(~(*sramPtr) & 0x0000FFFF);
        while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
        if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
        flashPtr++;
        *flashPtr =(uint16_t)((~(*sramPtr) >> 16) & 0x0000FFFF);
        while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
        if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
        flashPtr++;
        sramPtr++;
    }
//flashed up to 0x8007A00
//write first part of gfTCode into page 27(gProject==1) or page30(gProject==2)
    sramPtr = (uint32_t *) &(gfTCode[0]);	//WORD addressed!
    for (k = 0;k < 0x80;k++)  {				//128words=512bytes=0.5 page
        *flashPtr =(uint16_t)(~(*sramPtr) & 0x0000FFFF);
        while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
        if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
        flashPtr++;
        *flashPtr =(uint16_t)(~(*sramPtr >> 16) & 0x0000FFFF);
        while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
        if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
        flashPtr++;
        sramPtr++;
    }
    FLASH->CR &= ~FLASH_CR_PG;

//erase Page28 (gProject==1) or Page31 (gProject==2)
    FLASH->CR |= FLASH_CR_PER;
	if (gProject == 1) FLASH->AR = PAGE28;
	else FLASH->AR = PAGE31;
    FLASH->CR |= FLASH_CR_STRT;
    while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
    if ((FLASH->SR & FLASH_SR_EOP) !=0)  FLASH->SR |= FLASH_SR_EOP;
    FLASH->CR &= ~FLASH_CR_PER;

    FLASH->CR |= FLASH_CR_PG;
//write second half of gfTCode into  28(gProject==1) or page31(gProject==2)
	if (gProject == 1) flashPtr = (uint16_t *)PAGE28;
	else flashPtr = (uint16_t *)PAGE31;
    sramPtr = (uint32_t *)gfTCode + 0x80;	//start addr = 0x200bytes
    for (k = 0;k < 0x100;k++)  { //0x400bytes= 1 page, word read,halfWord burn
        *flashPtr =(uint16_t)(~(*sramPtr) & 0x0000FFFF);
        while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
        if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
        flashPtr++;
        *flashPtr =(uint16_t)(~(*sramPtr >> 16) & 0x0000FFFF);
        while ((FLASH->SR & FLASH_SR_BSY) != 0) ;
        if ((FLASH->SR & FLASH_SR_EOP) != 0)  FLASH->SR |= FLASH_SR_EOP;
        flashPtr++;
        sramPtr++;
    }
//flash is 100% filled
//terminate
    FLASH->CR &= ~FLASH_CR_PG;
//lock flash
    FLASH->CR |= FLASH_CR_LOCK;
    msDelay(10);
}

//###########################################################################

//USART Support
//*************
#ifdef CLOCK_48M
void initUsart1(void) // ****************************************************
// USART1-TxD is PortA,9
// USART1-RxD is PortA,10
// **************************************************************************
{
uint16_t baudset;

    RCC->APB2ENR |=  RCC_APB2ENR_USART1EN ;  // USART1 clock enable

#ifdef LQFP32
//set PA10 in alternate function mode
    GPIOA->MODER &= ~GPIO_MODER_MODER10_0 ;
    GPIOA->MODER |= GPIO_MODER_MODER10_1;
//set PB6 in alternate function mode
    GPIOB->MODER &= ~GPIO_MODER_MODER6_0 ;
    GPIOB->MODER |= GPIO_MODER_MODER6_1;
//USART1 is AF1 on PortA
    GPIOA->AFR[1] &= 0xFFFFF0FF;        //PA10 is RX
    GPIOA->AFR[1] |= 0x00000100;
//and AF0 on PortB
    GPIOB->AFR[0] &= 0xF0FFFFFF;        //PB6 is TX
    GPIOB->AFR[0] |= 0x00000000;
#else //TSSOP20
//set PA9 and PA10 in alternate function mode
    GPIOA->MODER &= ~(GPIO_MODER_MODER9_0 | GPIO_MODER_MODER10_0);
    GPIOA->MODER |= (GPIO_MODER_MODER9_1 | GPIO_MODER_MODER10_1);

//USART1 is AF1 on PortA
    GPIOA->AFR[1] &= 0xFFFFF00F;        //PA9 is TX, PA10 is RX
    GPIOA->AFR[1] |= 0x00000110;
#endif

    gUSART_RXHEAD = gUSART_RXBUF;
    gUSART_RXTAIL = gUSART_RXBUF;

   switch(gBaud) {
    case 96:
        baudset = 0x1388;       //9600Baud: 5000 at 48 MHz SysClock
        break;
    case 19:
        baudset = 0x09C4;       //19200Baud: 2500 at 48 MHz SysClock
        break;
    case 38:
        baudset = 0x04E2 ;      //38400Baud: 1250 at 48 MHz SysClock
        break;
    case 57:
        baudset = 0x0341;       //57600Baud: 833.33 at 48 MHz SysClock
        break;
    case 11:
    default:
        baudset = 0x01A1;       //112500Baud: 416.67 at 48 MHz SysClock
        break;
    }
    USART1->BRR = baudset;
	USART1->CR2 = USART_CR2_TXINV;	//1 Stopbit, TX level inverted
    USART1->CR3 = 0;				//default:no Handshake
    USART1->CR1 = USART_CR1_RXNEIE |USART_CR1_TE | USART_CR1_RE;
                                //8Bit, no Parity, RX Int
									//oversampling by 8 instead of 16
    USART1->CR1 |= USART_CR1_UE;	//enable USART
	gBaudSet = 1;
    NVIC_EnableIRQ(USART1_IRQn);
}

#else	

void initUsart1(void) // ****************************************************
// TSSOP20:USART1-RxD is PortA,9
// LQFP32:USART1-TxD is PortB,6
// USART1-RxD is PortA,10 in any case
// USART1 clock is always derived from SYSCLOCK=3.68MHz or 7.37MHz or14.74MHz
// **************************************************************************
{
uint16_t baudset;

    RCC->APB2ENR |=  RCC_APB2ENR_USART1EN ;   // USART1 clock enable

#ifdef LQFP32
//set PA10 in alternate function mode
    GPIOA->MODER &= ~GPIO_MODER_MODER10_0 ;
    GPIOA->MODER |= GPIO_MODER_MODER10_1;
//set PB6 in alternate function mode
    GPIOB->MODER &= ~GPIO_MODER_MODER6_0 ;
    GPIOB->MODER |= GPIO_MODER_MODER6_1;
//USART1 is AF1 on PortA
    GPIOA->AFR[1] &= 0xFFFFF0FF;        //PA10 is RX
    GPIOA->AFR[1] |= 0x00000100;
//and AF0 on PortB
    GPIOB->AFR[0] &= 0xF0FFFFFF;        //PB6 is TX
    GPIOB->AFR[0] |= 0x00000000;
#else //TSSOP20
//set PA9 and PA10 in alternate function mode
    GPIOA->MODER &= ~(GPIO_MODER_MODER9_0 | GPIO_MODER_MODER10_0);
    GPIOA->MODER |= (GPIO_MODER_MODER9_1 | GPIO_MODER_MODER10_1);

//USART1 is AF1 on PortA
    GPIOA->AFR[1] &= 0xFFFFF00F;        //PA9 is TX, PA10 is RX
    GPIOA->AFR[1] |= 0x00000110;
#endif
    gUSART_RXHEAD = gUSART_RXBUF;
    gUSART_RXTAIL = gUSART_RXBUF;

#ifdef CLOCK_3M68
    switch(gBaud) {
    case 96:
        baudset = 384;       //9600Baud: 384 at 3.6864MHz SYSCLK
        break;
    case 38:
        baudset = 96;       //38400Baud: 96 at 3.6864MHz SYSCLK
        break;
    case 19:
        baudset = 192;       //19200Baud: 192 at 3.6864MHz SYSCLK
        break;
    case 57:
        baudset = 64;       //57600Baud: 64 at 3.6864MHz SYSCLK
        break;
    case 11:
    default:
        baudset = 32;       //115200Baud: 32 at 3.6864MHz SYSCLK
        break;
    }
#endif

#ifdef CLOCK_14M
    switch(gBaud) {
    case 96:
        baudset = 1536;       //9600Baud: 1536 at 14.7456MHz SYSCLK
        break;
    case 38:
        baudset = 384;       //38400Baud: 384 at 14.7456MHz SYSCLK
        break;
    case 19:
        baudset = 768;       //19200Baud: 768 at 14.7456MHz SYSCLK
        break;
    case 57:
        baudset = 256;       //57600Baud: 256 at 14.7456MHz SYSCLK
        break;
    case 11:
    default:
        baudset = 128;       //115200Baud: 128 at 14.74564MHz SYSCLK
        break;
    }
#endif
    USART1->BRR = baudset;
	USART1->CR2 = USART_CR2_TXINV;	//1 Stopbit, TX level inverted
    USART1->CR3 = 0;				//default:no Handshake
    USART1->CR1 = USART_CR1_RXNEIE | USART_CR1_TE | USART_CR1_RE;
									//8Bit, no Parity, RX Int
    USART1->CR1 |= USART_CR1_UE;	//enable USART
    gBaudSet = 1;
    NVIC_EnableIRQ(USART1_IRQn);
}
#endif


void sendTB(uint8_t data) // *************************************************
//send 1 raw byte
// **************************************************************************
{
    while(!(USART1->ISR&USART_ISR_TXE));
    USART1->TDR = data;

	if (gLedBusy == 3) {
#ifdef LQFP32
  #ifdef LED_PB0
		GPIOB->BSRR = GPIO_BSRR_BR_0;	// clear bit PB0,LED OFF
  #endif
  #ifdef MODUL_24
		GPIOA->BSRR = GPIO_BSRR_BR_11;	// clear bit PA11,LED OFF
  #endif
  #ifdef LED_PB8
		GPIOB->BSRR = GPIO_BSRR_BR_8;	// clear bit PB8,LED OFF
  #endif
#else
  #ifdef MODUL_R
		GPIOB->BSRR = GPIO_BSRR_BR_1;	// clear bit PB1,LED OFF
  #else  //LED:PB8
		GPIOB->BSRR = GPIO_BSRR_BR_8;	// clear bit PB8,LED OFF
  #endif
#endif
		gLEDCOUNT = 4;					// ca.30 milliseconds
	}
}



void sendNibble(uint8_t data) // ********************************************
// **************************************************************************
{
uint8_t asciiNibble;

    asciiNibble = data+0x30;
    if (asciiNibble>0x39) asciiNibble+= 7;
	sendTB(asciiNibble);
}


void sendString(const uint8_t * str) // *************************************
// **************************************************************************
{
 uint8_t byte;

    while  ((byte = *str++) != 0)  sendTB(byte);
}


void sendX8(uint32_t data) // *********************************************
// send bits 0...7 in hex format (without leading 0x)
// **************************************************************************
{
    uint8_t temp;

    temp= (data & 0xF0) >> 4;
    temp = temp  +0x30;
    if (temp > 0x39) temp+= 7;
    sendTB(temp);
    temp= data & 0x0F;
    temp = temp + 0x30;
    if (temp > 0x39) temp += 7;
    sendTB(temp);
    sendTB(0x20);
}


void sendX32(uint32_t data) // **********************************************
// send 1 word (32bit) in hex format (without leading 0x)
// Leading zeroes are suppressed
// **************************************************************************
{
uint8_t nibble;
    nibble = (uint8_t) ((data>>28) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>24) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>20) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>16) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>12) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>8) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) ((data>>4) & 0x0000000F);
    sendNibble(nibble);
    nibble = (uint8_t) (data & 0x0000000F);
    sendNibble(nibble);
    sendTB(0x20);

}


void sendNX32(uint32_t data) // **********************************************
// send 1 word (32bit) in hex format (without leading 0x)
// **************************************************************************
{
uint8_t nibble, noLead;
    noLead = 0;
    nibble = (uint8_t) ((data>>28) & 0x0000000F);
    if (nibble)  {
        sendNibble(nibble);
        noLead = 1;
    }
    nibble = (uint8_t) ((data>>24) & 0x0000000F);
    if (nibble || noLead) {
        sendNibble(nibble);
        noLead = 1;
    }
    nibble = (uint8_t) ((data>>20) & 0x0000000F);
    if (nibble || noLead) {
        sendNibble(nibble);
        noLead = 1;
    }
    nibble = (uint8_t) ((data>>16) & 0x0000000F);
     if (nibble || noLead) {
        sendNibble(nibble);
        noLead = 1;
    }
    nibble = (uint8_t) ((data>>12) & 0x0000000F);
    if (nibble || noLead) {
        sendNibble(nibble);
        noLead = 1;
    }
    nibble = (uint8_t) ((data>>8) & 0x0000000F);
    if (nibble || noLead) {
        sendNibble(nibble);
        noLead = 1;
    }
    nibble = (uint8_t) ((data>>4) & 0x0000000F);
    if (nibble || noLead) {
        sendNibble(nibble);
        noLead = 1;
    }
    nibble = (uint8_t) (data & 0x0000000F);
    sendNibble(nibble);
}


void send0X32(uint32_t data) // **********************************************
// send 1 word (32bit) in hex format WITH leading 0x.Leading zeroes suppressed
// **************************************************************************
{
//	if (data >= 10)  {			//modified 11Jan24
		sendTB('0');
		sendTB('x');
//	}
	sendNX32(data);
}


void sendS32(uint32_t data) // *******************************************
//send signed decimal number string. Leading zeroes are suppressed
//trailing SPACE
//up to  +/- 2.147.483.647 Billions
// **************************************************************************
{
    sendNS32(data);
    sendTB(0x20);
}


 void sendLS32(uint32_t data) // ********************************************
//send signed decimal number string. Leading zeroes are suppressed
//leading SPACE
//up to  +/- 2.147.483.647 Billions
// **************************************************************************
{
    sendTB(0x20);
    sendNS32(data);
}


void sendNS32(uint32_t data) // *******************************************
//send signed decimal number string. Leading zeroes are suppressed
//no leading and trailing SPACE
//up to  +/- 2.147.483.647 Billions
// **************************************************************************
{
uint32_t quotient;

    if (data >= 0x80000000) {
        sendTB('-');
        data = ~data + 1 ;
    }
    if (data < 10) goto ONES;
    if (data < 100) goto TENS;
    if (data < 1000) goto HUNDREDS;
    if (data < 10000) goto THOUSANDS;
    if (data < 100000) goto TH10;
    if (data < 1000000) goto TH100;
    if (data < 10000000) goto MILLIONS;
    if (data < 100000000) goto MIL10;
    if (data < 1000000000) goto MIL100;
    quotient = data/1000000000;
    data = data - quotient*1000000000;
    sendTB((uint8_t)quotient+ 0x30);
MIL100:
    quotient = data/100000000;
    data = data - quotient*100000000;
    sendTB((uint8_t)quotient + 0x30);
MIL10:
    quotient = data/10000000;
    data = data - quotient*10000000;
    sendTB((uint8_t)quotient + 0x30);
MILLIONS:
    quotient = data/1000000;
    data = data - quotient*1000000;
    sendTB((uint8_t)quotient + 0x30);
TH100:
    quotient = data/100000;
    data = data - quotient*100000;
    sendTB((uint8_t)quotient+ 0x30);
TH10:
    quotient = data/10000;
    data = data - quotient*10000;
    sendTB((uint8_t)quotient + 0x30);
THOUSANDS:
    quotient = data/1000;
    data = data - quotient*1000;
    sendTB((uint8_t)quotient + 0x30);
HUNDREDS:
    quotient = data/100;
    data = data - quotient*100;
    sendTB((uint8_t)quotient+ 0x30);
TENS:
    quotient = data/10;
    data = data - quotient*10;
    sendTB((uint8_t)quotient + 0x30);
ONES:
    sendTB((uint8_t)data + 0x30) ;
}


uint32_t pullTermRxBuf(void) // ********************************************
// pull 1 byte asynchronously from USART reveive buffer
// check first if something received. If yes then set bit 15. If not return 0
// less overhead and more safe than use of a separate "buffer filled" flag
// **************************************************************************
{
uint32_t recv;

	servSystem();		//serve watchdog etc.

	if (gUSART_RXTAIL!=gUSART_RXHEAD)  {
		recv= (uint32_t)((*gUSART_RXTAIL++ & 0x000000FF)|0x00008000);
		if (gUSART_RXTAIL==gUSART_RXBUFTOP)  gUSART_RXTAIL=gUSART_RXBUF;

		if (gLedBusy > 1) {
#ifdef LQFP32
  #ifdef LED_PB0
			GPIOB->BSRR = GPIO_BSRR_BR_0;	// clear bit PB0,LED OFF
  #endif
  #ifdef MODUL_24
			GPIOA->BSRR = GPIO_BSRR_BR_11;	// clear bit PA11,LED OFF
  #endif
  #ifdef LED_PB8
			GPIOB->BSRR = GPIO_BSRR_BR_8;	// clear bit PB8,LED OFF
  #endif
#else
  #ifdef MODUL_R
			GPIOB->BSRR = GPIO_BSRR_BR_1;	// clear bit PB1,LED OFF
  #else		//LED_PB8
			GPIOB->BSRR = GPIO_BSRR_BR_8;	// clear bit PB8,LED OFF
  #endif
#endif
			gLEDCOUNT = 4;					//ca.300 milliseconds
		}
		return recv;
	}
	else return 0;
}


// Interrupt Handlers
//*******************

void SysTick_Handler(void)  // ***********************************************
// ***************************************************************************
{
	if (gSYSCOUNT) gSYSCOUNT--;			//controls servSystem()
	if (gDELAYCOUNT) gDELAYCOUNT--;
	if (gTIMEOUT) gTIMEOUT--;			//to be used in any subroutine
	if (gfMSCOUNT) gfMSCOUNT --;		//used by MS, DS
	if (gfTIXCOUNT) gfTIXCOUNT --;		//used by  TIX, WAIT
#ifdef MOTOR
	if (gfSTEPCOUNT > 0) gfSTEPCOUNT--;		//stepper timing
#endif
	gfRandom ^= gfRandom << 13;				//xorshift32 algorithm/white noise
	gfRandom ^= gfRandom >> 17;
	gfRandom ^= gfRandom << 5;
}

void USART1_IRQHandler(void)    // *****************************************
// *************************************************************************
{
uint32_t test;

	if (USART1->ISR & USART_ISR_RXNE)  {
		test = USART1->RDR & 0x00FF;
		if (test == CTRL_B) gfTermInput = CTRL_B;
		if (test != CTRL_D)  {
			*gUSART_RXHEAD = (uint8_t)test;
			gUSART_RXHEAD++;
			if (gUSART_RXHEAD==gUSART_RXBUFTOP)  gUSART_RXHEAD=gUSART_RXBUF;
		}
        else  op_DEBUG();
	}
}

// #########################################################################
